@@ -7,7 +7,6 @@ import contextlib
 import copy as _copy
 import dataclasses
 import functools
-import importlib.metadata
 import importlib.util
 import json
 import logging
@@ -93,12 +92,19 @@ __all__ = (
     "to_num",
     "breakdown_pydantic_annotation",
     "run_package_manager_command",
+    "StringEnum",
 )
 
 
 # --- General Global Utilities Types ---
 
 
+class StringEnum(str, Enum):
+    @classmethod
+    def allowed(cls) -> tuple[str, ...]:
+        return tuple(e.value for e in cls)
+
+
 class UndefinedType:
     def __init__(self) -> None:
         self.undefined = True
@@ -139,7 +145,7 @@ def hash_dict(data) -> int:
 
 class Params(BaseModel):
     def keys(self):
-        return self.model_fields.keys()
+        return type(self).model_fields.keys()
 
     def __call__(self, *args, **kwargs):
         raise NotImplementedError(
